#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdio.h>

#include "lcd.h"
#define FFT_N 256 // set to 256 point fft
#define LOG_OUT 1
#include "FFT.h" // include the library

void adc_init() {
   TIMSK0 = 0; // turn off timer0 for lower jitter

   ADCSRA = 0xe5; // set the adc to free running mode
   ADMUX = 0x40; // use adc0
}


int main() {

	adc_init();
	LCD_init();
   sei();

   while(1) { 
   
    for (int i = 0 ; i < 512 ; i += 2) { // save 256 samples
      while(!(ADCSRA & 0x10)); // wait for adc to be ready
      ADCSRA = 0xf5; // restart adc

      char m = ADCL; // fetch adc data
      char j = ADCH;
      int k = (j << 8) | m; // form into an int
      k -= 0x0200; // form into a signed int
      k <<= 6; // form into a 16b signed int

      fft_input[i] = k; // put real data into even bins
      fft_input[i+1] = 0; // set odd bins to 0
    }

    fft_window(); // window the data for better frequency response
    fft_reorder(); // reorder the data before doing the fft
    fft_run(); // process the data in the fft
    fft_mag_log(); // take the output of the fft
    
    
    int maxMagn = -9999;
    int maxIndex = -1;
    int err = 10;

    for(int i = 0; i < 128; i++) {
      if (fft_log_out[i] > maxMagn - err) {
         maxMagn = fft_log_out[i];
         maxIndex = i;
      }
    }
       

    char output[16];
    sprintf(output, "Freq: %d Hz", maxMagn); //tried: maxIndex * (sampleRate) / 256
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_print(output);
    _delay_ms(850);
    
  }
      
	return 0;
}